#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
test_brainiac
----------------------------------

tests for `brainiac` module.
"""

import unittest
import cloudpickle
from brainiac_client.scientist import package
import requests


class MockModel:
    def __init__(self, val: int):
        self.some_value = val

    def some_function(self, number: int):
        orig_some_val = self.some_value
        self.some_value = number
        return orig_some_val + number


class TestScientist(unittest.TestCase):
    def mockPost(self, *args, **kwargs):
        res = requests.Response()
        return res

    def setUp(self):
        self.og_put = requests.put
        self.og_post = requests.post
        requests.put = lambda _, data, **kwargs: kwargs.pop("files")
        requests.post = self.mockPost

    def tearDown(self):
        requests.put = self.og_put
        requests.post = self.og_post

    def test_package(self):
        request_dict = package(MockModel(42), "modelName")
        serialized_model = request_dict.get("pkledFile")
        model = cloudpickle.load(serialized_model)
        assert model.some_value == 42
        assert model.some_function(1) == 43
        assert model.some_value == 1



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: brainiac.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='brainiac.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0e\x62rainiac.proto\x1a\x1cgoogle/protobuf/struct.proto\".\n\x08UserData\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\"*\n\x0bProjectData\x12\r\n\x05owner\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"4\n\x06Status\x12\x13\n\x0bstatus_code\x18\x01 \x01(\x05\x12\x15\n\rstatus_reason\x18\x02 \x01(\t\"\x9f\x01\n\x11ThirdPartyRequest\x12\x19\n\x06\x63lient\x18\x01 \x01(\x0b\x32\t.UserData\x12\x10\n\x08\x66unction\x18\x02 \x01(\x0c\x12\x1e\n\x08projects\x18\x03 \x03(\x0b\x32\x0c.ProjectData\x12\x16\n\x0e\x66orce_download\x18\x04 \x01(\x08\x12%\n\x04\x62ody\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\"T\n\x12ThirdPartyResponse\x12\x17\n\x06status\x18\x01 \x01(\x0b\x32\x07.Status\x12%\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"\xa1\x01\n\x0f\x42rainiacRequest\x12\x19\n\x06\x63lient\x18\x01 \x01(\x0b\x32\t.UserData\x12\x1d\n\x07project\x18\x02 \x01(\x0b\x32\x0c.ProjectData\x12\x15\n\rfunction_name\x18\x03 \x01(\t\x12\x16\n\x0e\x66orce_download\x18\x04 \x01(\x08\x12%\n\x04\x62ody\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\"R\n\x10\x42rainiacResponse\x12\x17\n\x06status\x18\x01 \x01(\x0b\x32\x07.Status\x12%\n\x04\x62ody\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct2\xb7\x01\n\x06\x45ngine\x12\x34\n\x0brunFunction\x12\x10.BrainiacRequest\x1a\x11.BrainiacResponse\"\x00\x12?\n\x12thirdPartyFunction\x12\x12.ThirdPartyRequest\x1a\x13.ThirdPartyResponse\"\x00\x12\x36\n\rlistFunctions\x12\x10.BrainiacRequest\x1a\x11.BrainiacResponse\"\x00\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,])




_USERDATA = _descriptor.Descriptor(
  name='UserData',
  full_name='UserData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='username', full_name='UserData.username', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='UserData.password', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=48,
  serialized_end=94,
)


_PROJECTDATA = _descriptor.Descriptor(
  name='ProjectData',
  full_name='ProjectData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='owner', full_name='ProjectData.owner', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='ProjectData.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=96,
  serialized_end=138,
)


_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status_code', full_name='Status.status_code', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status_reason', full_name='Status.status_reason', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=140,
  serialized_end=192,
)


_THIRDPARTYREQUEST = _descriptor.Descriptor(
  name='ThirdPartyRequest',
  full_name='ThirdPartyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client', full_name='ThirdPartyRequest.client', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='function', full_name='ThirdPartyRequest.function', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='projects', full_name='ThirdPartyRequest.projects', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='force_download', full_name='ThirdPartyRequest.force_download', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='body', full_name='ThirdPartyRequest.body', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=195,
  serialized_end=354,
)


_THIRDPARTYRESPONSE = _descriptor.Descriptor(
  name='ThirdPartyResponse',
  full_name='ThirdPartyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='ThirdPartyResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='body', full_name='ThirdPartyResponse.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=356,
  serialized_end=440,
)


_BRAINIACREQUEST = _descriptor.Descriptor(
  name='BrainiacRequest',
  full_name='BrainiacRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client', full_name='BrainiacRequest.client', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='project', full_name='BrainiacRequest.project', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='function_name', full_name='BrainiacRequest.function_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='force_download', full_name='BrainiacRequest.force_download', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='body', full_name='BrainiacRequest.body', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=443,
  serialized_end=604,
)


_BRAINIACRESPONSE = _descriptor.Descriptor(
  name='BrainiacResponse',
  full_name='BrainiacResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='BrainiacResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='body', full_name='BrainiacResponse.body', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=606,
  serialized_end=688,
)

_THIRDPARTYREQUEST.fields_by_name['client'].message_type = _USERDATA
_THIRDPARTYREQUEST.fields_by_name['projects'].message_type = _PROJECTDATA
_THIRDPARTYREQUEST.fields_by_name['body'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_THIRDPARTYRESPONSE.fields_by_name['status'].message_type = _STATUS
_THIRDPARTYRESPONSE.fields_by_name['body'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_BRAINIACREQUEST.fields_by_name['client'].message_type = _USERDATA
_BRAINIACREQUEST.fields_by_name['project'].message_type = _PROJECTDATA
_BRAINIACREQUEST.fields_by_name['body'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_BRAINIACRESPONSE.fields_by_name['status'].message_type = _STATUS
_BRAINIACRESPONSE.fields_by_name['body'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
DESCRIPTOR.message_types_by_name['UserData'] = _USERDATA
DESCRIPTOR.message_types_by_name['ProjectData'] = _PROJECTDATA
DESCRIPTOR.message_types_by_name['Status'] = _STATUS
DESCRIPTOR.message_types_by_name['ThirdPartyRequest'] = _THIRDPARTYREQUEST
DESCRIPTOR.message_types_by_name['ThirdPartyResponse'] = _THIRDPARTYRESPONSE
DESCRIPTOR.message_types_by_name['BrainiacRequest'] = _BRAINIACREQUEST
DESCRIPTOR.message_types_by_name['BrainiacResponse'] = _BRAINIACRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

UserData = _reflection.GeneratedProtocolMessageType('UserData', (_message.Message,), dict(
  DESCRIPTOR = _USERDATA,
  __module__ = 'brainiac_pb2'
  # @@protoc_insertion_point(class_scope:UserData)
  ))
_sym_db.RegisterMessage(UserData)

ProjectData = _reflection.GeneratedProtocolMessageType('ProjectData', (_message.Message,), dict(
  DESCRIPTOR = _PROJECTDATA,
  __module__ = 'brainiac_pb2'
  # @@protoc_insertion_point(class_scope:ProjectData)
  ))
_sym_db.RegisterMessage(ProjectData)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), dict(
  DESCRIPTOR = _STATUS,
  __module__ = 'brainiac_pb2'
  # @@protoc_insertion_point(class_scope:Status)
  ))
_sym_db.RegisterMessage(Status)

ThirdPartyRequest = _reflection.GeneratedProtocolMessageType('ThirdPartyRequest', (_message.Message,), dict(
  DESCRIPTOR = _THIRDPARTYREQUEST,
  __module__ = 'brainiac_pb2'
  # @@protoc_insertion_point(class_scope:ThirdPartyRequest)
  ))
_sym_db.RegisterMessage(ThirdPartyRequest)

ThirdPartyResponse = _reflection.GeneratedProtocolMessageType('ThirdPartyResponse', (_message.Message,), dict(
  DESCRIPTOR = _THIRDPARTYRESPONSE,
  __module__ = 'brainiac_pb2'
  # @@protoc_insertion_point(class_scope:ThirdPartyResponse)
  ))
_sym_db.RegisterMessage(ThirdPartyResponse)

BrainiacRequest = _reflection.GeneratedProtocolMessageType('BrainiacRequest', (_message.Message,), dict(
  DESCRIPTOR = _BRAINIACREQUEST,
  __module__ = 'brainiac_pb2'
  # @@protoc_insertion_point(class_scope:BrainiacRequest)
  ))
_sym_db.RegisterMessage(BrainiacRequest)

BrainiacResponse = _reflection.GeneratedProtocolMessageType('BrainiacResponse', (_message.Message,), dict(
  DESCRIPTOR = _BRAINIACRESPONSE,
  __module__ = 'brainiac_pb2'
  # @@protoc_insertion_point(class_scope:BrainiacResponse)
  ))
_sym_db.RegisterMessage(BrainiacResponse)



_ENGINE = _descriptor.ServiceDescriptor(
  name='Engine',
  full_name='Engine',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=691,
  serialized_end=874,
  methods=[
  _descriptor.MethodDescriptor(
    name='runFunction',
    full_name='Engine.runFunction',
    index=0,
    containing_service=None,
    input_type=_BRAINIACREQUEST,
    output_type=_BRAINIACRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='thirdPartyFunction',
    full_name='Engine.thirdPartyFunction',
    index=1,
    containing_service=None,
    input_type=_THIRDPARTYREQUEST,
    output_type=_THIRDPARTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='listFunctions',
    full_name='Engine.listFunctions',
    index=2,
    containing_service=None,
    input_type=_BRAINIACREQUEST,
    output_type=_BRAINIACRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_ENGINE)

DESCRIPTOR.services_by_name['Engine'] = _ENGINE

# @@protoc_insertion_point(module_scope)



from typing import List, Any, Dict


def action(models: List[Any], body: Dict) -> List[int]:
    ans = []
    for idx, model in enumerate(models):
        func_name = body["function"]
        function = getattr(model, func_name)
        ans.append(function(body))
    return ans



from io import BytesIO
from typing import Any, List, Optional
import pkg_resources
import json
import os
import urllib.parse
from brainiac_client.constants import (PICKLE_PROTOCOL, BRAINIAC_CLIENT_PASSWORD,
                                       BRAINIAC_CLIENT_USERNAME)
import cloudpickle
import requests


def login(url, username: Optional[str], password: Optional[str]) -> requests.Response:
    login_url = urllib.parse.urljoin(url, "/api/login")
    params = {'username': username, 'password': password}
    res = requests.post(url=login_url, json=params)
    return res


def get_installed_packages():
    installed_packages = pkg_resources.working_set
    return [i.key for i in installed_packages]


def package(some_class: Any,
            modelName: str,
            url: str = "http://libnosis.com",
            dependencies: List[str] = []) -> requests.Response:

    res = login(url, os.environ.get(BRAINIAC_CLIENT_USERNAME),
                os.environ.get(BRAINIAC_CLIENT_PASSWORD))
    cookies = res.headers.get("Set-Cookie")

    file = BytesIO(cloudpickle.dumps(some_class, protocol=PICKLE_PROTOCOL))

    files = {"pkledFile": file}
    data = {"modelName": modelName, "dependencies": json.dumps(dependencies)}
    url = urllib.parse.urljoin(url, "/api/model/upload")
    response = requests.put(url, files=files, data=data, headers={"Cookie": cookies})
    return response


__all__ = ["package"]



PICKLE_PROTOCOL = 4
BRAINIAC_CLIENT_USERNAME = "BRAINIAC_CLIENT_USERNAME"
BRAINIAC_CLIENT_PASSWORD = "BRAINIAC_CLIENT_PASSWORD"
DEFAULT_PORT = 5000
BRAINIAC_ENGINE_URL = "https://brainiac-engine.herokuapp.com/"



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import brainiac_client.protos.brainiac_pb2 as brainiac__pb2


class EngineStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.runFunction = channel.unary_unary(
        '/Engine/runFunction',
        request_serializer=brainiac__pb2.BrainiacRequest.SerializeToString,
        response_deserializer=brainiac__pb2.BrainiacResponse.FromString,
        )
    self.thirdPartyFunction = channel.unary_unary(
        '/Engine/thirdPartyFunction',
        request_serializer=brainiac__pb2.ThirdPartyRequest.SerializeToString,
        response_deserializer=brainiac__pb2.ThirdPartyResponse.FromString,
        )
    self.listFunctions = channel.unary_unary(
        '/Engine/listFunctions',
        request_serializer=brainiac__pb2.BrainiacRequest.SerializeToString,
        response_deserializer=brainiac__pb2.BrainiacResponse.FromString,
        )


class EngineServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def runFunction(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def thirdPartyFunction(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def listFunctions(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_EngineServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'runFunction': grpc.unary_unary_rpc_method_handler(
          servicer.runFunction,
          request_deserializer=brainiac__pb2.BrainiacRequest.FromString,
          response_serializer=brainiac__pb2.BrainiacResponse.SerializeToString,
      ),
      'thirdPartyFunction': grpc.unary_unary_rpc_method_handler(
          servicer.thirdPartyFunction,
          request_deserializer=brainiac__pb2.ThirdPartyRequest.FromString,
          response_serializer=brainiac__pb2.ThirdPartyResponse.SerializeToString,
      ),
      'listFunctions': grpc.unary_unary_rpc_method_handler(
          servicer.listFunctions,
          request_deserializer=brainiac__pb2.BrainiacRequest.FromString,
          response_serializer=brainiac__pb2.BrainiacResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'Engine', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import unittest
import mock
import os
import cloudpickle
from brainiac_client.client import BrainiacClient
from brainiac_client.constants import BRAINIAC_CLIENT_USERNAME, BRAINIAC_CLIENT_PASSWORD
from brainiac_client.protos.brainiac_pb2 import (BrainiacRequest, BrainiacResponse, Status,
                                                 ThirdPartyRequest, ThirdPartyResponse)
from brainiac_client.tests.third_party import action
from google.protobuf.struct_pb2 import Struct


class MockStub:
    def __init__(self, channel):
        # self.runFunction = self.runfunc
        pass

    def runFunction(self, request: BrainiacRequest) -> BrainiacResponse:
        body = Struct()
        body.update({
            "response": {
                "function_name": request.function_name,
                "force_download": request.force_download,
                "project": {
                    "owner": request.project.owner,
                    "name": request.project.name
                },
                "client": {
                    "username": request.client.username,
                    "password": request.client.password
                },
                "body": request.body["number"]
            }
        })
        return BrainiacResponse(status=Status(status_code=200), body=body)

    def listFunctions(self, request: BrainiacRequest) -> BrainiacResponse:
        body = Struct()
        body.update({"response": ["some_function"]})
        return BrainiacResponse(status=Status(status_code=200), body=body)

    def thirdPartyFunction(self, request: ThirdPartyRequest) -> ThirdPartyResponse:
        body = Struct()
        body.update({
            "response": {
                "function":
                cloudpickle.loads(request.function).__name__,
                "force_download":
                request.force_download,
                "project": [{
                    "owner": project.owner,
                    "name": project.name
                } for project in request.projects],
                "client": {
                    "username": request.client.username,
                    "password": request.client.password
                },
                "body":
                request.body["number"]
            }
        })
        return ThirdPartyResponse(status=Status(status_code=200), body=body)


class TestServer(unittest.TestCase):
    @mock.patch.dict(os.environ, {
        BRAINIAC_CLIENT_USERNAME: "username",
        BRAINIAC_CLIENT_PASSWORD: "password"
    })
    def test_CreateFunctionGrpc(self):
        stub = MockStub(None)
        client = BrainiacClient(http=False, stub=stub)
        model = client.get_model_client("testUser0", "testProject0")
        response = model.some_function({"number": 10})
        expected_response = Struct()
        expected_response.update({
            "response": {
                "function_name": "some_function",
                "force_download": False,
                "project": {
                    "owner": "testUser0",
                    "name": "testProject0"
                },
                "client": {
                    "username": 'username',
                    "password": 'password'
                },
                "body": 10
            }
        })
        assert response == expected_response

    @mock.patch.dict(os.environ, {
        BRAINIAC_CLIENT_USERNAME: "username",
        BRAINIAC_CLIENT_PASSWORD: "password"
    })
    def test_ThirdPartyFunctionGrpc(self):
        stub = MockStub(None)
        client = BrainiacClient(http=False, stub=stub)
        model = client.get_model_client("testUser0", "testProject0")
        response = client.runThirdPartyFunction([model], action, {"number": 10})
        expected_response = Struct()
        expected_response.update({
            "response": {
                "function": "action",
                "force_download": False,
                "project": [{
                    "owner": "testUser0",
                    "name": "testProject0"
                }],
                "client": {
                    "username": "username",
                    "password": "password"
                },
                "body": 10
            }
        })
        assert response == expected_response



import atexit
import os
from typing import Dict, List, Optional, Callable
from urllib.parse import urljoin
import cloudpickle

import grpc
import requests
from google.protobuf.struct_pb2 import Struct

from brainiac_client.errors import AuthenticationError
from brainiac_client.protos.brainiac_pb2 import (BrainiacRequest, BrainiacResponse, ProjectData,
                                                 ThirdPartyRequest, ThirdPartyResponse, UserData)
from brainiac_client.protos.brainiac_pb2_grpc import EngineStub
import brainiac_client.constants as constants


class ModelClient:
    def __init__(self, grpc_stub, client, project, force_download, url):
        self.stub = grpc_stub
        self.client = client
        self.project = project
        self.force_download = force_download
        self.url = url

    def set_methods(self, methods: List[str], http):
        if http:
            [setattr(self, method, self.__create_http_method(method)) for method in methods]
        else:
            [setattr(self, method, self.__create_grpc_method(method)) for method in methods]

    def __create_http_method(self, name: str):
        def wrapper(body: Dict):
            struct = Struct()
            struct.update(body)
            br_req = BrainiacRequest(client=self.client,
                                     project=self.project,
                                     function_name=name,
                                     body=struct,
                                     force_download=self.force_download)
            url = urljoin(self.url, "/runFunction")
            res = requests.get(url, data=br_req.SerializeToString())
            response = BrainiacResponse.FromString(res.content)
            if response.status.status_code == 200:
                return response.body
            raise Exception("{}:{}".format(response.status.status_code,
                                           response.status.status_reason))

        return wrapper

    def __create_grpc_method(self, name: str):
        def wrapper(body: Dict):
            struct = Struct()
            struct.update(body)
            response = self.stub.runFunction(
                BrainiacRequest(client=self.client,
                                project=self.project,
                                function_name=name,
                                body=struct,
                                force_download=self.force_download))
            if response.status.status_code == 200:
                return response.body
            return None

        return wrapper


class BrainiacClient:
    def __init__(self, url: Optional[str] = None, stub: EngineStub = None, http=True):
        if url is None:
            url = constants.BRAINIAC_ENGINE_URL if http else 'localhost:{}'.format(
                constants.DEFAULT_PORT)
        self.url = url
        self.channel = grpc.insecure_channel(url)
        self.stub = stub or EngineStub(self.channel)
        self.http = http
        self.client = UserData(username=os.environ.get(constants.BRAINIAC_CLIENT_USERNAME),
                               password=os.environ.get(constants.BRAINIAC_CLIENT_PASSWORD))
        atexit.register(self.channel.close)

    def __make_request(self, url: str, request_object, response_type):
        res = requests.get(url, data=request_object.SerializeToString())
        return response_type.FromString(res.content)

    def get_model_client(self, owner: str, project: str,
                         force_download=False) -> Optional[ModelClient]:
        project_data = ProjectData(owner=owner, name=project)
        br_req = BrainiacRequest(client=self.client, project=project_data)
        response = None
        if self.http:
            url = urljoin(self.url, "/listFunctions")
            response = self.__make_request(url, br_req, BrainiacResponse)
        else:
            response = self.stub.listFunctions(br_req)
        if response.status.status_code == 200:
            methods = response.body["response"]
            model = ModelClient(self.stub, self.client, project_data, force_download, self.url)
            model.set_methods(methods, self.http)
            return model
        elif response.status.status_code == 404:
            return None
        elif response.status.status_code == 401:
            raise AuthenticationError(response.status.status_reason)
        else:
            raise Exception(response.status.status_reason)

    def runThirdPartyFunction(self,
                              models: List[ModelClient],
                              function: Callable,
                              data: Dict,
                              force_download=False) -> Dict:
        body = Struct()
        body.update(data)
        thpr = ThirdPartyRequest(client=self.client,
                                 projects=[model.project for model in models],
                                 function=cloudpickle.dumps(function,
                                                            protocol=constants.PICKLE_PROTOCOL),
                                 body=body,
                                 force_download=force_download)
        response = None
        if self.http:
            url = urljoin(self.url, "/thirdPartyFunction")
            response = self.__make_request(url, thpr, ThirdPartyResponse)
        else:
            response = self.stub.thirdPartyFunction(thpr)
        if response.status.status_code != 200:
            raise Exception(response.status.status_reason)
        return response.body



#!/usr/bin/env python
"""A setuptools based setup module for brainiac"""
# -*- coding: utf-8 -*-

from codecs import open
from os import path
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as readme_file:
    readme = readme_file.read()

with open(path.join(here, 'HISTORY.rst'), encoding='utf-8') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

with open(path.join(here, "requirements.txt")) as f:
    requirements = [line.replace(",", "") for line in f.read().split("\n") if line]

print(requirements)

setup(
    name='brainiac_client',
    version="0.0.1.7-preview",
    description="The client library to make and use brainiac modules.",
    # long_description=readme + '\n\n' + history,
    author="Suyog Soti",
    author_email='suyog.soti@gmail.com',
    url='https://github.com/suyogsoti/client-lib',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    include_package_data=True,
    install_requires=requirements,
    license="MIT",
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=requirements,
)
